@@ -12,7 +12,9 @@
 #'   (this part will be perfomed before character substitutions by \code{subst})
 #' @param subst (character of length=2, or matrix with 2 columns) pair(s) of character-strings for replacement (1st as search-item and 2nd as replacement); this part is performed after character-removal via \code{rem} 
 #' @param lstE (character, length=1) names of list-elements where colnames should be cleaned
+#' @param mathOper (character, length=1) optional mathematical operation on numerical part of sample-names (eg \code{mathOper='/2'} for deviding numeric part of colnames by 2)
 #' @param silent (logical) suppress messages
+#' @param debug (logical) additional messages for debugging
 #' @param callFrom (character) allow easier tracking of messages produced
 #' @seealso \code{\link[base]{grep}}  
 #' @return This function returns a list (equivalent to input \code{dat})
@@ -21,9 +23,11 @@
 #' dat1 <- list(raw=dat1, quant=dat1, notes="other..") 
 #' cleanListCoNames(dat1, rem=c(left="sample_"), c(".","-")) 
 #' @export
-cleanListCoNames <- function(dat, rem=NULL, subst=c("-","_"), lstE=c("raw","quant","counts"), silent=FALSE, callFrom=NULL) {
+cleanListCoNames <- function(dat, rem=NULL, subst=c("-","_"), lstE=c("raw","quant","counts"), mathOper=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
   ## clean/stratify columnames
   fxNa <- wrMisc::.composeCallName(callFrom, newNa="cleanListCoNames")
+  if(!isTRUE(silent)) silent <- FALSE
+  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
   ch1 <- lstE %in% names(dat)
   if(!is.list(dat) | all(!ch1)) {if(!silent) message(fxNa," Nothing to do (verifiy your input)"); ok <- FALSE} else ok <- TRUE 
   if(ok & any(!ch1)) { if(!silent) message(fxNa," Term ",wrMisc::pasteC(lstE[which(!ch1)],quoteC="'")," not found in 'dat', ignoring ..")
@@ -41,6 +45,8 @@ cleanListCoNames <- function(dat, rem=NULL, subst=c("-","_"), lstE=c("raw","quan
     if(any(ch1)) rem[which(ch1)] <- paste0(rem[which(ch1)],"$")
     for(i in lstE) for(j in rem) colnames(dat[[i]]) <- sub(j,"",colnames(dat[[i]]))
   }
+  if(debug) { message(fxNa,"cLC1")}
+  
   if(ok & length(subst) >0) {
     ## character substitution
     if(length(dim(subst)) >1) if(any(dim(subst) < 1:2)) { subst <- NULL
@@ -53,5 +59,24 @@ cleanListCoNames <- function(dat, rem=NULL, subst=c("-","_"), lstE=c("raw","quan
       for(i in lstE) 
       colnames(dat[[i]]) <- sub(subst[1], subst[2], colnames(dat[[i]]))}
   }
+  if(debug) { message(fxNa,"cLC2")}
+
+  if(ok & length(mathOper)==1) if(nchar(mathOper) <2) { mathOper <- NULL
+    message(fxNa,"Invalid entry for 'mathOper'  ... ignoring" )}
+  if(ok & length(mathOper)==1) {
+    iniNa <- colnames(dat[[lstE[1]]])
+    txNa <- sub("^[[:space:]]*[[:digit:]]+","", iniNa)   # remove heading numeric part
+    if(debug) { message(fxNa,"cLC3"); cLC3 <- list(dat=dat,lstE=lstE,iniNa=iniNa,txNa=txNa)}
+    if(all(nchar(txNa) < nchar(iniNa))) {
+      nuNa <- substr(iniNa, 1, nchar(iniNa) -nchar(txNa))            # the (supposed) numeric part
+      if(debug) message(fxNa," mathematical transformation on colnames by  ",mathOper,"  on  ",wrMisc::pasteC(utils::head(nuNa),lastCol=", "))     
+      nuNa2 <- try(eval(parse(text=paste0("c(",paste(nuNa, collapse=","),")",mathOper))), silent=TRUE)
+      if(inherits(nuNa2, "try-error") | !is.numeric(nuNa2)) { if(!silent) message(fxNa,"Can't run mathematical operation, ",
+        "heading part in column-heads might not to be true numeric or 'mathOper' somehow incorrect !")
+      } else {
+        newNa <- paste0(nuNa2, txNa)
+        for(i in lstE) if(length(dim(dat[[i]])) >1) colnames(dat[[i]]) <- newNa else if(!silent) message(fxNa,"Note '$",i,"' seems not to fit") }
+    } else if(!silent) message(fxNa,"Some column-names seem not to contain any numeric part at beginning, nothing to do ...") 
+  }
   dat }
   
