@@ -1,20 +1,19 @@
 #' Read Tabulated Files Exported by FragPipe At Protein Level
 #'
-#' Protein identification and quantification results form \href{https://fragpipe.nesvilab.org/}{Fragpipe}
-#' which were exported as tabulated text can be imported and relevant information extracted.
-#' The final output is a list containing 3 elements: \code{$annot}, \code{$raw} and optional \code{$quant}, or returns data.frame with entire content of file if \code{separateAnnot=FALSE}.
+#' This function allos importing protein identification and quantification results form \href{https://fragpipe.nesvilab.org/}{Fragpipe}
+#' which were previously exported as tabulated text (tsv). Quantification data and other relevant information will be extracted similar like the other import-functions from this package.
+#' The final output is a list containing the elements: \code{$annot}, \code{$raw} and \code{$quant}, or a data.frame with the quantication data and a part of the annotation if argument \code{separateAnnot=FALSE}.
 #'
 #' @details
-#' This function has been developed using Fragpipe versions 18.0.
-#' The
-#' Using the argument \code{suplAnnotFile} it is possible to specify a specific file (or search for default file) to read for extracting file-names as sample-names and other experiment realted information.
-#' This function replaces the depreciated function \code{readPDExport}.
+#' This function has been developed using Fragpipe versions 18.0 and 19.0.
+#'
+#' Using the argument \code{suplAnnotFile} it is possible to specify a specific file (or search for default file) to read for extracting file-names as sample-names and other experiment related information.
 #'
 #' @param fileName (character) name of file to be read
 #' @param path (character) path of file to be read
 #' @param normalizeMeth (character) normalization method, defaults to \code{median}, for more details see \code{\link[wrMisc]{normalizeThis}})
 #' @param sampleNames (character) new column-names for quantification data (ProteomeDiscoverer does not automatically use file-names from spectra); this argument has priority over \code{suplAnnotFile}
-#' @param read0asNA (logical) decide if initial quntifications at 0 should be transformed to NA
+#' @param read0asNA (logical) decide if initial quntifications at 0 should be transformed to NA (thus avoid -Inf in log2 results)
 #' @param quantCol (character or integer) exact col-names, or if length=1 content of \code{quantCol} will be used as pattern to search among column-names for $quant using \code{grep}
 #' @param refLi (character or integer) custom specify which line of data is main species, if character (eg 'mainSpe'), the column 'SpecType' in $annot will be searched for exact match of the (single) term given
 #' @param separateAnnot (logical) if \code{TRUE} output will be organized as list with \code{$annot}, \code{$abund} for initial/raw abundance values and \code{$quant} with final log2 (normalized) quantitations
@@ -26,11 +25,12 @@
 #' @param specPref (character or list) define characteristic text for recognizing (main) groups of species (1st for comtaminants - will be marked as 'conta', 2nd for main species- marked as 'mainSpe',
 #'  and optional following ones for supplemental tags/species - maked as 'species2','species3',...);
 #'  if list and list-element has multiple values they will be used for exact matching of accessions (ie 2nd of argument \code{annotCol})
-
 #' @param gr (character or factor) custom defined pattern of replicate association, will override final grouping of replicates from \code{sdrf} and/or \code{suplAnnotFile} (if provided)   \code{}
 #' @param sdrf (character, list or data.frame) optional extraction and adding of experimenal meta-data: if character, this may be the ID at ProteomeExchange. Besides, the output from \code{readSdrf} or a list from \code{defineSamples} may be provided; if \code{gr} is provided, it gets priority for grouping of replicates
 #' @param suplAnnotFile (logical or character) optional reading of supplemental files; however, if \code{gr} is provided, \code{gr} gets priority for grouping of replicates;
 #'  if \code{character} the respective file-name (relative or absolute path)
+#' @param groupPref (list) additional parameters for interpreting meta-data to identify structure of groups (replicates), will be passed to \code{readSampleMetaData}.
+#'   May contain \code{lowNumberOfGroups=FALSE} for automatically choosing a rather elevated number of groups if possible (defaults to low number of groups, ie higher number of samples per group)
 #' @param plotGraph (logical or integer) optional plot of type vioplot of initial and normalized data (using \code{normalizeMeth}); if integer, it will be passed to \code{layout} when plotting
 #' @param silent (logical) suppress messages
 #' @param debug (logical) additional messages for debugging
@@ -49,15 +49,9 @@
 #' @export
 readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sampleNames=NULL, read0asNA=TRUE, quantCol="Intensity$",
   annotCol=NULL,  refLi=NULL, separateAnnot=TRUE, FDRCol=list("Protein.Probability", lim=0.99),    # contamCol="Contaminant",
-  plotGraph=TRUE, tit="Fragpipe", graphTit=NULL, wex=1.6, specPref=c(conta="CON_|LYSC_CHICK", mainSpecies="OS=Homo sapiens"),
+  groupPref=list(lowNumberOfGroups=TRUE), plotGraph=TRUE, tit="FragPipe", graphTit=NULL, wex=1.6, specPref=c(conta="CON_|LYSC_CHICK", mainSpecies="OS=Homo sapiens"),
   gr=NULL, sdrf=NULL, suplAnnotFile=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
 
- # path=NULL;  normalizeMeth="median";  sampleNames=NULL;  read0asNA=TRUE;  quantCol="Intensity$";
- # annotCol=NULL;   refLi=NULL;  separateAnnot=TRUE;  FDRCol=list("Protein.Probability", lim=0.99);     # contamCol="Contaminant";
- # plotGraph=TRUE;  tit="Fragpipe";  graphTit=NULL;  wex=1.6;  specPref=c(conta="CON_|LYSC_CHICK", mainSpecies="OS=Homo sapiens");
- # gr=NULL;  sdrf=NULL;  suplAnnotFile=FALSE;  silent=FALSE;  debug=FALSE;  callFrom=NULL
-
-
   ## read Fragpipe exported txt
   fxNa <- wrMisc::.composeCallName(callFrom, newNa="readFragpipeFile")
   oparMar <- if(plotGraph) graphics::par("mar") else NULL       # only if figure might be drawn
@@ -75,7 +69,7 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
   msg <- "Invalid entry for 'fileName'"
   if(length(fileName) >1) { fileName <- fileName[1]
     if(!silent) message(fxNa," 'fileName' shoud be of length=1, using 1st value")
-  } else { if(length(fileName) <1) stop(msg) else if(nchar(fileName) <0) stop(msg)}
+  } else { if(length(fileName) <1) stop(msg) else if(is.na(fileName) | nchar(fileName) <1) stop(msg)}
   paFi <- fileName                      # presume (& correct if path is given)
   chFi <- file.exists(fileName)         # presume (& correct otherwise)
   if(length(path) >0) if(!dir.exists(path[1])) { path <- NULL
@@ -83,7 +77,7 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
   if(length(path) >0) { chFi <- file.exists(file.path(path[1], fileName))
     if(chFi) paFi <- file.path(path[1], fileName) else {
       if(file.exists(fileName)) {paFi <- fileName
-        if(!silent) message(fxNa,"NOTE : Unable to find file '",fileName,"' in path '",path,"' but found without specified path !")
+        if(!silent) message(fxNa,"Note : Unable to find file '",fileName,"' in path '",path,"' but found without specified path !")
       } else chFi <- FALSE                      # if path+fileName not found, check without path
   } }
   if(!chFi) stop(" File ",fileName," was NOT found ",if(length(path) >0) paste(" in path ",path)," !")
@@ -143,11 +137,45 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
     tmp <- .chColNa(annotCol[3], tmp, renameTo="EntryName", silent=silent, fxNa=fxNa)           # like  THOC2_MOUSE
     tmp <- .chColNa(annotCol[4], tmp, renameTo="Description", silent=silent, fxNa=fxNa)         # full (long) name
 
-    #annot <- tmp[,wrMisc::naOmit(annotColNo)]
     annot <- cbind(Accession=tmp[,"Accession"], EntryName=tmp[,"EntryName"], GeneName=NA, Species=NA, Contam=NA, SpecType=NA,
       Description=tmp[,"Description"], tmp[,wrMisc::naOmit(annotColNo[-(1:6)])])   # may be better to name column 'species'
-    if(debug) { message(fxNa,"rfp2 .. annotColNo : ", wrMisc::pasteC(annotColNo))
-      rfp2 <- list()}
+    if(debug) { message(fxNa,"rfp2 .. annotColNo : ", wrMisc::pasteC(annotColNo)); rfp2 <- list(annot=annot,annotCol=annotCol,tmp=tmp,specPref=specPref )}
+
+    ## Species  (need to run before reparsing nadly parsed)
+    if(!is.na(annotColNo[6])) { spec <- tmp[,annotColNo[6]]
+      spec <- sub("^\ +|\ +$","",spec)          # remove heading or tailing (white) space
+      chOX <- grep(" OX=", spec)
+      if(length(chOX) >0) { OX <- sub(" OX=", "", spec[chOX])
+        spec[chOX] <- sub(" OX=[[:digit:]]+[[:print:]]*","", spec[chOX])
+        chO2 <- nchar(spec[chOX]) <3 & nchar(OX) >1
+        if(any(chO2)) spec[chOX[which(chO2)]] <- OX[which(chO2)]    # use OX=.. in case no other information available
+      }
+      if(TRUE) spec <- sub(" \\([[:alpha:]][[:print:]]+\\).*", "", spec)   # remove ' (..)'
+      annot[,"Species"] <- spec
+    }
+
+    ## look for not well parsed (use separator '|' as indicator)
+    chPa <- grep("\\|", annot[,"Accession"])
+    if(length(chPa) >0) {
+      chSp <- grep(" ", annot[chPa,"Accession"])
+      if(length(chSp) >0) {
+        # extract species
+        chOS <- grep("[[:print:]]+ OS=[[:alpha:]]", annot[chPa[chSp],"Accession"])
+        if(length(chOS) >0) annot[chPa[chSp[chOS]],"Species"] <- sub(" [[:upper:]]{2}=.+","", sub("[[:print:]]+ OS=","", annot[chPa[chSp[chOS]],"Accession"]))   # extract species
+        ## extract GeneName
+        chGn <- grep("[[:print:]]+ GN=", annot[chPa[chSp],"Accession"])
+        if(length(chGn) >0) annot[chPa[chSp[chGn]],"GeneName"] <- sub(" [[:upper:]]{2}=.+","", sub("[[:print:]]+ GN=","", annot[chPa[chSp[chGn]],"Accession"]))
+        ## extract Description
+        annot[chPa[chSp],"Description"] <-  sub(".*? ", "", sub(" [[:upper:]]{2}=.+","", annot[chPa[chSp],"Accession"]))
+        ## extract EntryName (option 1)
+        annot[chPa[chSp],"EntryName"] <- gsub(".*\\|","", sub(" .+","", annot[chPa,"Accession"]))
+      } else {
+        annot[chPa,"EntryName"] <- gsub(".*\\|","", annot[chPa,"Accession"])     ## extract EntryName (option 2)
+      }
+      ## extract Accession
+      annot[chPa,"Accession"] <- sapply(strsplit(annot[chPa,"Accession"], "\\|"), function(x) if(length(x) >1) x[2] else NA)
+    }
+
 
     ## clean 'Description' entries: remove tailing punctuation or open brackets (ie not closed) at end of (truncated) fasta header
     if(cleanDescription) {
@@ -167,29 +195,6 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
       if(length(useLi) >0) annot[useLi,"Contam"] <- toupper(gsub(" ","",annot[useLi,"Contaminant"]))}
     chConta <- grep("^contam", tmp[,annotCol[1]])                    # specific to  Fragpipe
     if(length(chConta) >0) annot[chConta,"Contam"] <- TRUE
-    ## look for tags from  specPref
-
-    if(length(specPref) >0) {
-      ## get index where specPref are located
-      if("conta" %in% names(specPref)) {
-        chConta <- unlist(lapply(annotCol[c(1,3:5)], function(x) if(x %in% colnames(tmp)) grep(specPref["conta"], tmp[,x])))
-        if(length(chConta) >0) { chConta <- sort(unique(chConta))
-          annot[chConta, "Contam"] <- TRUE } }
-    } else if(debug) message(fxNa,"Note: Argument 'specPref' not specifed (empty)")
-    if(debug) {message(fxNa,"rfp6b ..  ")}
-
-    ## Species
-    if(!is.na(annotColNo[6])) { spec <- tmp[,annotColNo[6]]
-      spec <- sub("^\ +|\ +$","",spec)          # remove heading or tailing (white) space
-      chOX <- grep(" OX=", spec)
-      if(length(chOX) >0) { OX <- sub(" OX=", "", spec[chOX])
-        spec[chOX] <- sub(" OX=[[:digit:]]+[[:print:]]*","", spec[chOX])
-        chO2 <- nchar(spec[chOX]) <3 & nchar(OX) >1
-        if(any(chO2)) spec[chOX[which(chO2)]] <- OX[which(chO2)]    # use OX=.. in case no other information available
-      }
-      annot[,"Species"] <- spec
-    }
-    if(debug) {message(fxNa,"rfp6c ..  ")}
 
     ## get more species annot;  separate multi-species (create columns 'Accession','GeneName','Species','SpecType')
     chSp <- is.na(annot[,"Species"]) | nchar(annot[,"Species"]) <2
@@ -202,7 +207,14 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
         if(length(chSp3) >0) for(i in chSp3) annot[chSep,"Species"] <- commonSpec[i,2]
       }
       chSp <- is.na(annot[,"Species"]) | nchar(annot[,"Species"]) <2 }     # update
-    if(debug) {message(fxNa,"rfp6d ..  ")}
+    if(debug) {message(fxNa,"rfp6d ..  "); rfp6d <- list(annot=annot,tmp=tmp,chSp=chSp,specPref=specPref,annotCol=annotCol,PSMCol=PSMCol,PepCol=PepCol)}
+
+    ## look for tags from  specPref
+    if(length(specPref) >0) {
+      ## set annot[,"specPref"] according to specPref
+      annot <- .extrSpecPref(specPref, annot, silent=silent, debug=debug, callFrom=fxNa)
+    } else if(debug) message(fxNa,"Note: Argument 'specPref' not specifed (empty)")
+    if(debug) {message(fxNa,"rfp6b ..  ")}
 
     if(!silent) {
       if(any(chSp, na.rm=TRUE) & !all(chSp)) message(fxNa,"Note: ",sum(chSp)," (out of ",nrow(tmp),") lines with unrecognized species")
@@ -286,32 +298,47 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
       if(length(PSMCol) >0) counts[,,"PSM"] <- as.matrix(tmp[,PSMCol])
       if(length(PepCol) >0) counts[,,"UniquePeptides"] <- as.matrix(tmp[,PepCol])
     } else counts <- NULL
-    if(debug) {message(fxNa,"rfp12 .. ")}
+    if(debug) {message(fxNa,"rfp12 .. ");
+      rfp12 <- list(tmp=tmp,abund=abund,annot=annot,sdrf=sdrf, fileName=fileName,path=path,paFi=paFi,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            refLi=refLi,specPref=specPref,read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol,gr=gr) }
+
 
     ## check for reference for normalization
     refLiIni <- refLi
     if(is.character(refLi) & length(refLi)==1) { refLi <- which(annot[,"SpecType"]==refLi)
       if(length(refLi) <1 & !silent) message(fxNa,"Could not find any protein matching argument 'refLi', ignoring ...") else {
         if(!silent) message(fxNa,"Normalize using (custom) subset of ",length(refLi)," lines specified as '",refLiIni,"'")}}    # may be "mainSpe"
+    if(length(refLi) <1) refLi <- 1:nrow(abund)
+
+    ## set 0 values to NA (avoid -Inf at log2)
+    if(!isFALSE(read0asNA)) { ch0 <- abund ==0
+      if(any(ch0, na.rm=TRUE)) abund[which(ch0)] <- NA }
+
     ## take log2 & normalize
     quant <- if(utils::packageVersion("wrMisc") > "1.10") {
         try(wrMisc::normalizeThis(log2(abund), method=normalizeMeth, mode="additive", refLines=refLi, silent=silent, callFrom=fxNa), silent=TRUE)
       } else try(wrMisc::normalizeThis(log2(abund), method=normalizeMeth, refLines=refLi, silent=silent, callFrom=fxNa), silent=TRUE)       #
-    if(debug) { message(fxNa,"rfp13 .. dim quant: ", nrow(quant)," li and  ",ncol(quant)," cols; colnames : ",wrMisc::pasteC(colnames(quant))," ")}
+    if(debug) { message(fxNa,"rfp13 .. dim quant: ", nrow(quant)," li and  ",ncol(quant)," cols; colnames : ",wrMisc::pasteC(colnames(quant))," ")
+      rfp13 <- list(tmp=tmp,quant=quant,abund=abund,annot=annot,sdrf=sdrf, fileName=fileName,path=path,paFi=paFi,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            refLi=refLi,specPref=specPref,read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol,gr=gr) }
 
     ### GROUPING OF REPLICATES AND SAMPLE META-DATA
     if(any((!isFALSE(suplAnnotFile) & length(suplAnnotFile) >0), length(sdrf) >0)) {
-      setupSd <- readSampleMetaData(sdrf=sdrf, suplAnnotFile=suplAnnotFile, quantMeth="PD", path=path, abund=utils::head(quant), silent=silent, debug=debug, callFrom=fxNa)
+      setupSd <- readSampleMetaData(sdrf=sdrf, suplAnnotFile=suplAnnotFile, quantMeth="FP", path=path, abund=utils::head(quant), groupPref=groupPref, silent=silent, debug=debug, callFrom=fxNa)
     }
-    if(debug) {message(fxNa,"rfp13b")}
+    if(debug) {message(fxNa,"rfp13b"); rfp13b <- list(tmp=tmp,quant=quant,abund=abund,setupSd=setupSd,annot=annot,sdrf=sdrf, fileName=fileName,path=path,paFi=paFi,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            specPref=specPref,read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol, gr=gr )}
 
     ## finish groups of replicates & annotation setupSd
     if(length(setupSd) >0 & length(setupSd$groups) <1) {                       # if nothing found/matching from sdrf & file, try getting sample-setup from colnames (use if at least 1 replicate found)
       if(length(gr) ==ncol(abund)) setupSd$groups <- gr else {
         if(debug) {message(fxNa,"rfp13e  Note: setupSd$groups is still empty ! ")}
         if(length(setupSd$lev) ==ncol(abund)) setupSd$groups <- setupSd$lev else {
+          if(length(setupSd$lev) >0 & ncol(abund) >0) warning(fxNa," sdrf sample-metadata dot not fit (sdrf suggests ",length(setupSd$lev)," but real data consist of ",ncol(abund)," samples) !")
           ## try defining groups based on colnames
-          if(debug) {message(fxNa,"rfp13f  Note: setupSd is still empty !  .. try getting sample-setup from colnames")}
+          if(debug) {message(fxNa,"rfp13f  Note: setupSd$lev is still empty/not matching !  .. try getting sample-setup from colnames");
+            rfp13f <- list(tmp=tmp,quant=quant,abund=abund,setupSd=setupSd,annot=annot,sdrf=sdrf, ileName=fileName,path=path,paFi=paFi,tmp=tmp,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol )}
           delPat <- "_[[:digit:]]+$|\\ [[:digit:]]+$|\\-[[:digit:]]+$"       # remove enumerators, ie trailing numbers after separator
           grou <- sub(delPat,"", colnames(abund))
           if(length(unique(grou)) >1 & length(unique(grou)) < ncol(abund)) setupSd$groups <- grou else {
@@ -319,18 +346,28 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
             if(length(unique(grou)) >1 & length(unique(grou)) < ncol(abund)) setupSd$groups <- grou }
         } }
     }
+    if(debug) {message(fxNa,"rfp13g"); rfp13g <- list(tmp=tmp,quant=quant,abund=abund,setupSd=setupSd,annot=annot,sdrf=sdrf, ileName=fileName,path=path,paFi=paFi,tmp=tmp,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol )}
 
     ## finish sample-names: use file-names from meta-data if no custom 'sampleNames' furnished
     ## One more check for colnames & sampleNames
+    if(length(sampleNames) <1) sampleNames <- wrMisc::trimRedundText(
+       if(length(setupSd$sdrfDat$comment.data.file.) ==ncol(abund)) setupSd$sdrfDat$comment.data.file. else colnames(abund), minNchar=2, spaceElim=TRUE, silent=silent, callFrom=fxNa, debug=debug)
+
+    if(debug) {message(fxNa,"rfp13h"); rfp13h <- list(tmp=tmp,quant=quant,abund=abund,setupSd=setupSd,annot=annot,sdrf=sdrf, fileName=fileName,path=path,paFi=paFi,tmp=tmp,normalizeMeth=normalizeMeth,sampleNames=sampleNames,
+            read0asNA=read0asNA,quantCol=quantCol,annotCol=annotCol,refLi=refLi,separateAnnot=separateAnnot,FDRCol=FDRCol )}
+
     if(length(sampleNames) == ncol(quant)) {
       colnames(quant) <- colnames(abund) <- sampleNames     # use custom provided sampleNames
       if(length(dim(counts)) >1 & length(counts) >0) colnames(counts) <- sampleNames
     } else {
-      if(debug) { message(fxNa,"rfp13c") }
-      sampleNames <- if(length(setupSd$annotBySoft$File.Name)==ncol(quant)) {    # try taking sampleNames from sdrf
-        basename(sub("\\.raw$|\\.Raw$|\\.RAW$","", setupSd$annotBySoft$File.Name)) } else colnames(abund)     # otherwise remain with colnames from main file
-      sampleNames <- wrMisc::trimRedundText(sampleNames, minNchar=2, spaceElim=TRUE, silent=silent, callFrom=fxNa, debug=debug)
-      colnames(quant) <- colnames(abund) <- sampleNames
+      if(debug) { message(fxNa,"rfp13i") }
+      if("annotBySoft$File.Name" %in% names(setupSd)) {
+        sampleNames <- if(length(setupSd$annotBySoft$File.Name)==ncol(quant)) {    # try taking sampleNames from sdrf
+          basename(sub("\\.raw$|\\.Raw$|\\.RAW$","", setupSd$annotBySoft$File.Name)) } else colnames(abund)     # otherwise remain with colnames from main file
+        sampleNames <- wrMisc::trimRedundText(sampleNames, minNchar=2, spaceElim=TRUE, silent=silent, callFrom=fxNa, debug=debug)
+        colnames(quant) <- colnames(abund) <- sampleNames
+      } else  colnames(quant) <- sampleNames <- colnames(abund)
       if(length(dim(counts)) >1 & length(counts) >0) colnames(counts) <- sampleNames
 
     }
@@ -343,7 +380,9 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
     if(plotGraph) {
       if(debug) {message(fxNa,"rfp15 .. length custLay ", length(custLay) )}
       if(length(custLay) >0) graphics::layout(custLay) else {if(!identical(normalizeMeth,"none") & length(quant) >0) graphics::layout(1:2)}
-      graphics::par(mar=c(3, 3, 3, 1))                          # mar: bot,le,top,ri
+      ch1 <- try(graphics::par(mar=c(3, 3, 3, 1)), silent=TRUE)                          # mar: bot,le,top,ri
+      if(inherits(ch1, "try-error")) {plotGraph <- FALSE; message(fxNa,"UNABLE to graphical paramters !! Abandoning plot ! check current graphics device ...")} }      
+    if(plotGraph) {
       if(length(graphTit) >0) message(fxNa,"Argument 'graphTit' is depreciated, please rather use 'tit'")
       if(is.null(tit) & !is.null(graphTit)) tit <- graphTit     # for derpreciated argument
       if(is.null(tit)) tit <- "Fragpipe quantification "
@@ -364,14 +403,16 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
           graphics::abline(h=round(stats::median(quant, na.rm=TRUE)) +c(-2:2)*3, lty=2, col=grDevices::grey(0.6)) }
       } else {                                            # wrGraph and sm are available
         if(debug) {message(fxNa,"rfp18  print vioplotW "  )}
-        wrGraph::vioplotW(log2(abund), tit=paste(tit,"(initial)",sep=" "), wex=wex, silent=silent, callFrom=fxNa)
-        graphics::abline(h=round(stats::median(log2(abund), na.rm=TRUE)) +c(-2:2)*3, lty=2, col=grDevices::grey(0.6))
+        ch1 <- try(wrGraph::vioplotW(log2(abund), tit=paste(tit,"(initial)",sep=" "), wex=wex, silent=silent, callFrom=fxNa), silent=TRUE)
+        if(inherits(ch1, "try-error")) {plotGraph <- FALSE; message(fxNa,"UNABLE to plot vioplotW !!")
+        } else graphics::abline(h=round(stats::median(log2(abund), na.rm=TRUE)) +c(-2:2)*3, lty=2, col=grDevices::grey(0.6))
         ## now normalized
         if(debug) {message(fxNa,"rfp19  print norm vioplotW() ",identical(normalizeMeth,"none")," ou ", length(quant) <0)}
         if(!identical(normalizeMeth,"none") | length(quant) >0) {
           if(debug) {message(fxNa,"rfp20  print vioplotW() for normalized")}
-            wrGraph::vioplotW(quant, tit=paste(tit,", ",normalizeMeth,"-normalized",titSu), wex=wex, silent=silent, callFrom=fxNa)
-            graphics::abline(h=round(stats::median(quant, na.rm=TRUE)) +c(-2:2)*3, lty=2, col=grDevices::grey(0.6)) }
+            ch1 <- try(wrGraph::vioplotW(quant, tit=paste(tit,", ",normalizeMeth,"-normalized",titSu), wex=wex, silent=silent, callFrom=fxNa), silent=TRUE)
+            if(inherits(ch1, "try-error")) {plotGraph <- FALSE; message(fxNa,"UNABLE to plot vioplotW !!")
+            } else graphics::abline(h=round(stats::median(quant, na.rm=TRUE)) +c(-2:2)*3, lty=2, col=grDevices::grey(0.6)) }
         on.exit(graphics::par(mar=oparMar)) }            # restaure old settings
     }
 
@@ -382,4 +423,4 @@ readFragpipeFile <- function(fileName, path=NULL, normalizeMeth="median", sample
     ## final output
     if(isTRUE(separateAnnot)) list(raw=abund, quant=quant, annot=annot, counts=counts, sampleSetup=setupSd, quantNotes=parametersD, notes=notes) else data.frame(quant,annot) }
 }
-  
+
